pub enum InsertValue {
Prepend(u64),
PrependReplace(u64, LinkOffset),
Tombstone,
TombstonePrefix,
}Expand description
Specify value to insert. Used by insert_advanced.
Variants
Prepend(u64)
Insert as a head of the existing linked list.
PrependReplace(u64, LinkOffset)
Replace the linked list. Then insert as a head.
Tombstone
Effectively delete associated values for the specified key.
TombstonePrefix
Effectively delete associated values for all keys starting with the prefix.