Module log

Append-only storage with indexes and integrity checks.

See Log for the main structure. This module also provides surrounding types needed to construct the Log, including IndexDef and some iterators.

Structs

FlushFilterContext
Potentially useful context for the flush filter function.
FoldDef
Definition of a “fold” function.
IndexDef
Definition of an index. It includes: name, function to extract index keys, and how much the index can lag on disk.
Log
An append-only storage with indexes and integrity checks.
LogIter
Iterator over all entries in a Log.
LogLookupIter
Iterator over Log entries selected by an index lookup.
LogMetadata
Metadata about index names, logical [Log] and [Index] file lengths.
LogRangeIter
Iterator over keys and LogLookupIter, filtered by an index prefix.
OpenOptions
Options used to configured how an Log is opened.
Wait
State to detect on-disk updates.

Enums

ChecksumType
What checksum function to use for an entry.
FlushFilterOutput
Output of a flush filter.
GenericPath
Abstract Path for [Log].
IndexOutput
Output of an index function. Bytes that can be used for lookups.

Traits

Fold
The actual logic of a “fold” function, and its associated state.

Type Aliases

FlushFilterFunc