pub enum IndexOutput {
Reference(Range<u64>),
Owned(Box<[u8]>),
Remove(Box<[u8]>),
RemovePrefix(Box<[u8]>),
}Expand description
Output of an index function. Bytes that can be used for lookups.
Variants
Reference(Range<u64>)
The index key is a slice, relative to the data entry (ex. input of the index function).
Use this if possible. It generates smaller indexes.
Owned(Box<[u8]>)
The index key is a separate sequence of bytes unrelated to the input bytes.
Use this if the index key is not in the entry. For example, if the entry is compressed.
Remove(Box<[u8]>)
Remove all values associated with the key in the index.
This only affects the index. The entry is not removed in the log.
RemovePrefix(Box<[u8]>)
Remove all values associated with all keys with the given prefix in the index.
This only affects the index. The entry is not removed in the log.