Struct RotateLog

pub struct RotateLog { /* private fields */ }
A collection of Logs that get rotated or deleted automatically when they exceed size or count limits.

Writes go to the active Log. Reads scan through all Logs.

Implementations

Source

impl RotateLog

pub fn append(&mut self, data: impl AsRef<[u8]>) -> Result<()>

Append data to the writable Log.

pub fn lookup( &self, index_id: usize, key: impl Into<Bytes>, ) -> Result<RotateLogLookupIter<'_>>

Look up an entry using the given index. The index_id is the index of index_defs stored in OpenOptions.

pub fn slice_to_bytes(&self, slice: &[u8]) -> Bytes

Convert a slice to Bytes.

Do not copy the slice if it’s from the main on-disk buffer of one of the loaded logs.

pub fn lookup_latest( &self, index_id: usize, key: impl AsRef<[u8]>, ) -> Result<LogLookupIter<'_>>

Look up an entry using the given index. The index_id is the index of index_defs stored in OpenOptions.

Unlike RotateLog::lookup, this function only checks the “latest” (i.e. “writable”) Log without checking others. It is useful to make sure certain contents depending on other entries are inserted into the same Log.

Practically, a flush_filter should also be used to make sure dependent entries are stored in a same Log. So this function will panic if flush_filter is not set on OpenOptions.

pub fn sync(&mut self) -> Result<u8>

Read latest data from disk. Write in-memory entries to disk.

Return the index of the latest Log.

For in-memory RotateLog, this function always returns 0.

pub fn remove_old_logs(&mut self) -> Result<()>

Attempt to remove outdated logs.

Does nothing if the content of the ‘latest’ file has changed on disk, which indicates rotation was triggered elsewhere, or the RotateLog is in-memory.

pub fn is_changed_on_disk(&self) -> bool

Returns true if sync will load more data on disk.

This function is optimized to be called frequently. It does not access the filesystem directly, but communicate using a shared mmap buffer.

This is not about testing buffered pending changes. To access buffered pending changes, use RotateLog::iter_dirty instead.

pub fn flush(&mut self) -> Result<u8>

Renamed. Use RotateLog::sync instead.

pub fn iter(&self) -> impl Iterator<Item = Result<&[u8]>>

Iterate over all the entries.

The entries are returned in FIFO order.

pub fn iter_dirty(&self) -> impl Iterator<Item = Result<&[u8]>>

Iterate over all dirty entries.