Module utils

Functions

atomic_read
Read the entire file written by atomic_write.
atomic_write
Atomically create or replace a file with the given content. Attempt to use symlinks on unix if SYMLINK_ATOMIC_WRITE is set.
atomic_write_plain
Atomically create or replace a file with the given content. Use a plain file. Do not use symlinks.
mmap_bytes
Return a read-only view of the entire file.
mmap_path
Similar to mmap_bytes, but accepts a Path directly so the callsite does not need to open a File.
open_dir
Open a path. Usually for locking purpose.
xxhash
xxhash32