pub fn open_dir(lock_path: impl AsRef<Path>) -> Result<File>Expand description
Open a path. Usually for locking purpose.
The path is assumed to be a directory. But this function does not do extra checks to make sure. If path is not a directory, this function might still succeed on unix systems.
Windows does not support opening a directory. This function will create a file called “lock” inside the directory and open that file instead.